# DBrb - database access library for Ruby.
# 
# Copyright (c) 2006 Tim Becker
#
# Released under the same terms as Ruby
# (http://www.ruby-lang.org/en/LICENSE.txt)
#
#
# see DBrb for documentation.

require 'dbi'


#:title: DBrb : RDoc Documenation

# +DBrb+ is  a wrapper to facilitate working with {Ruby
# DBI}[http://sourceforge.net/projects/ruby-dbi].  The intention is to
# make the most commonly used DB functionality (as determined by me) easy and
# intuitive to use.
#
# === Examples
#
# 	db = DBrb.new("my_dbi_driver_string", "usr", "credentials")
#
# 	# return single value or array if a single row is selected
#	i = db.sql("select max(id) from some_table") 
#	name, last_name =  db.sql("SELECT name, last_name FROM some_name WHERE id = ?", 1000)
#
# should the two examples above return more than one row, the first
# one is used.
# 
#	db.sql("SELECT name, last_name FROM some_name WHERE id < ?", 1000) do |row|
#		puts "#{row.name} #{row.last_name}"
#		# in case of conflicts with existing methods, you can use:
#		row["last_name"]
#	end
#	
# The row that the +sql+ iterates over is the standard DBI::Row, which
# has been supplemented with accessor methods named after the columns
# in case these don't conflict with existing methods.


class DBrb

	# 
	def initialize driver, usr=nil, pwd=nil, params=nil
		@driver=driver
		@usr=usr
		@pwd=pwd
		@params=params
		

		@pstatements = {}

		if block_given?
			yield self
			self.close
		end

	end
	
	# +sql+ is the method that does all the work. The only required
	# parameter is +stmt+, which passes a string with the SQL
	# statement to be executed. The optional parameters in +args+
	# get bound to '?'-placeholders in the statement.
	#
	# This method can be called in several ways.
	# * Statements not returning results:
	#
	#	db.sql "UPDATE sometable SET somevalue=somevalue+1"
	#
	# * If the statement returns a single value, that value is returned by the method call:
	# 
	#
	#	value = db.sql "SELECT value FROM sometable LIMIT 1"
	#
	# * Statements returning several values in a single row:
	#	first_name, last_name = 
	#		db.sql "SELECT first, last 
	#			FROM sometable
	#			LIMIT 1"
	#
	# (It should go without saying: if a single row result is expected like in the
	# examples above but the statement returns several rows, the first one is used.)
	#
	# * If a block is passed to the method, it can be used to iterate over the results.
	#
	#	db.sql("SELECT first, last FROM sometable WHERE last=?", "Smith") do |row|
	#		puts "Name: #{row.first} #{row.last}
	#	end  
	#
	# * or, in case the resulting rows only contain a single column :
	# 	
	#	db.sql "SELECT first FROM sometable" do |firstname|
	#		puts "Hello #{firstname}!"
	#	end
	
	def sql(stmt, *args, &block) #:yields: row
				#optional args and an optional block
		_sql_internal(stmt,false,*args, &block)
	end

	# Similar to the sql method. This method is meant for `INSERT`
	# and `UPDATE` statements and returns the RPC (row processed
	# count), i.e. the number of rows affected by the statement.
	# 

	def sql_count (stmt, *args, &block)
		_sql_internal(stmt,true,*args,&block)	
	end

	# Closes all resources used by this object instance.
	def close 
		@pstatements.each_value {|pstmt|
			pstmt[0].finish
		}
		@pstatements = {}
		@db.disconnect
		@db = nil
	end

	# _Debug_ _method_. This method dumps statistics about how often
	# each sql statement was executed by this instance. Statistics
	# are reset after calls to `close`
	def dumpStmtStats 
		@pstatements.each_pair { |stmt, arr|
			puts "#{stmt} : #{arr[1]}"
		} if @pstatements
	end

	def to_s
		"DBrb: #{@driver}"
	end

protected

	# Returns a (cached) reference to the DBI driver.  Default
	# behaviour is to create a single connection, keeping it cached
	# and reusing it for all DB statements executed by this object
	# instance.
	#
	# This method is used internally by `sql` and could be
	# overridden in case you require different behaviour, e.g.
	# creating a new connection for each executed statement or
	# maintaining several connections.
	
	def get_db 
		return @db if @db && @db.connected?  
		@db = DBI.connect(@driver, @usr, @pwd, @params) 
	end

	def _sql_internal (stmt, count, *args) #:yields: row
		#optional args and an optional block
		pstmt = get_pstmt stmt
		pstmt.execute *args
		
		
		ret_val = nil	
		begin
			if block_given?
				pstmt.each { |row|
					if row.length==1
						yield row[0]
					else
						yield row
					end
				} 
			else 
					unless count
						ret_val = pstmt.fetch if pstmt.fetchable?
						ret_val = ret_val[0] if ret_val && ret_val.length==1
					end
		
			end	
		rescue NoMethodError 
			# calling fetch for mysql drivers on stmts returning
			# no rows (INSERT...) is fucked.

#			Message: <"undefined method `fetch_row' for nil:NilClass">
#			---Backtrace---
#			/usr/local/lib/ruby/site_ruby/1.8/DBD/Mysql/Mysql.rb:424:in `fetch'
#			/usr/local/lib/ruby/site_ruby/1.8/dbi/dbi.rb:811:in `fetch'
#			/usr/local/lib/ruby/site_ruby/1.8/dbi/dbi.rb:836:in `each'
		end
	
		ret_val = pstmt.rows if count
		pstmt.cancel # clean up
		return ret_val

	end


	
private
	
	# Maintains a cache of all prepared statments.  Each prepared
	# statement executed by `sql` is prepared only once and reused in
	# case the same sql statement is executed a second time.
	#
	# Prepared statements are disposed when `close` is called.
	
	def get_pstmt stmt #:doc:
		#@pstatements[stmt] ||= get_db.prepare stmt
		@pstatements[stmt] ||= [nil,0]
		@pstatements[stmt][1] += 1
		@pstatements[stmt][0] ||= get_db.prepare stmt
	end
	

end
module DBI
	# augments DBI::Row with a +method_missing+ method that lets the
	# row behave as if it had accessor methods for each colum.	
	class Row
		def method_missing meth_id
			return self[meth_id]
		end
	end
end

if $0==__FILE__
	sql = DBrb.new('DBI:PG:stats2', 'test', 'test')
	row = sql.sql("SELECT * FROM post WHERE nick=? LIMIT 10", "muppet")
	puts row.nick
	row = sql.sql("SELECT * FROM post WHERE nick=? LIMIT 10", "Philo")
	puts row.nick
	puts sql.sql("SELECT count(*) from post")
	
	puts sql.sql("SELECT count(*) from post")

	puts sql.sql("SELECT nick from post limit 20") {|nick|
		puts nick
	}

	puts sql.sql("SELECT nick, url from post limit 20") {|row|
		puts row.nick, row.url
	}
	
	sql.dumpStmtStats
	
	sql.close

	

end
